﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace TestThinkGeoMap
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private RectangleShape maxExtent;
        ThinkGeo.MapSuite.WpfDesktopEdition.WpfMap wpfMap;
        LayerOverlay restrictionOverlay = null;
        private RectangleShape maximumExtent = new RectangleShape(-0.5, 0.5, 100.5, -100.5);
        public MainWindow()
        {
            InitializeComponent();
            wpfMap = new ThinkGeo.MapSuite.WpfDesktopEdition.WpfMap();
            wpfMap.Background = Brushes.Black;
            wpfMap.MapUnit = ThinkGeo.MapSuite.Core.GeographyUnit.Meter;
            wpfMap.Visibility = System.Windows.Visibility.Visible;
            wpfMap.Width = 800;
            wpfMap.Height = 600;

            this.layoutGrid.Children.Add(wpfMap);
        }

        void wpfMap_OverlayDrawing(object sender, OverlayDrawingWpfMapEventArgs e)
        {
            RectangleShape currentOverlayExtent = e.Overlay.MapArguments.CurrentExtent;
            double currentOverlayScale = e.Overlay.MapArguments.CurrentScale;
        }

        void wpfMap_CurrentExtentChanging(object sender, CurrentExtentChangingWpfMapEventArgs e)
        {
            //double minLp = e.NewExtent.GetBoundingBox().LowerLeftPoint.X;
            //double minLat = e.NewExtent.GetBoundingBox().LowerLeftPoint.Y;
            //double maxLon = e.NewExtent.GetBoundingBox().UpperRightPoint.X;
            //double maxLat = e.NewExtent.GetBoundingBox().UpperRightPoint.Y;

            //if ((minLp < maxExtent.LowerLeftPoint.X) || (minLat < maxExtent.LowerLeftPoint.Y) || (maxLon > maxExtent.UpperRightPoint.X) || (maxLat > maxExtent.UpperRightPoint.Y))
            //{
            //	e.Cancel = true;
            //}
            //else
            //{
            //	e.Cancel = false;
            //}

            if (e.CurrentExtent.GetBoundingBox().Width != e.NewExtent.GetBoundingBox().Width)
            {

                if (e.NewExtent.IsWithin(this.maximumExtent) == false)
                {

                    e.Cancel = true;
                    wpfMap.CurrentExtent = this.maximumExtent;

                }

            }

            else
            {

                RectangleShape currentIntersection = maximumExtent.GetIntersection(e.CurrentExtent);

                RectangleShape newIntersection = maximumExtent.GetIntersection(e.NewExtent);

                //Gets the area. Meter is an arbitrary unit.

                double currentArea = currentIntersection.GetArea(GeographyUnit.Meter, AreaUnit.SquareMeters);

                double newArea = newIntersection.GetArea(GeographyUnit.Meter, AreaUnit.SquareMeters);



                if (newArea <= currentArea)
                {

                    e.Cancel = true;
                    e.CurrentExtent = maximumExtent;
                    //wpfMap.RestrictExtent = this.maxExtent;

                }

            }


        }

        void layerOverLay_DrawingException(object sender, DrawingExceptionTileOverlayEventArgs e)
        {
            throw new NotImplementedException();
        }

        void layerOverLay_Drawing(object sender, DrawingOverlayEventArgs e)
        {

        }

        private sealed class RetainableMemoryStream : MemoryStream
        {
            internal RetainableMemoryStream()
            {
            }

            internal bool AllowDisposition { get; set; }

            protected override void Dispose(bool disposing)
            {
                if (this.AllowDisposition || (!disposing))
                {
                    base.Dispose(disposing);
                }
            }
        }

        private void layoutButton_Click(object sender, RoutedEventArgs e)
        {
            RestrictionLayer restrictionLayer = new RestrictionLayer();
            restrictionLayer.Transparency = 1.0f;
            restrictionLayer.Zones.Add(maximumExtent);
            restrictionLayer.RestrictionMode = RestrictionMode.ShowZones;
            restrictionLayer.UpperScale = wpfMap.MaximumScale;
            restrictionLayer.LowerScale = double.MinValue;
            wpfMap.MapResizeMode = MapResizeMode.PreserveScaleAndCenter;
            this.layoutButton.IsEnabled = false;
            this.removeButton.IsEnabled = true;
            //this.wpfMap.CurrentScale = 1.0;
            //wpfMap.CurrentExtent = maximumExtent;
            //wpfMap.RestrictExtent = maximumExtent;
            //wpfMap.RestrictExtent = new RectangleShape(-0.5, 0.5, 100.5, -100.5); 
            //maxExtent = wpfMap.CurrentExtent;
            //wpfMap.CurrentExtentChanging += wpfMap_CurrentExtentChanging;
            //wpfMap.RestrictExtent = wpfMap.CurrentExtent;
            //wpfMap.MaximumScale = wpfMap.CurrentScale;
            //wpfMap.RestrictExtent = new RectangleShape(-0.5, 0.5, 50, -200.5);
            //wpfMap.CurrentExtent = new RectangleShape(-0.5, 0.5, 100, -767.5);

            restrictionOverlay = new LayerOverlay();
            restrictionOverlay.Layers.Add("RestrictionLayer", restrictionLayer);



            wpfMap.Overlays.Add("RestrictionOverlay", restrictionOverlay);


            this.wpfMap.Overlays["RestrictionOverlay"].Refresh();
            this.wpfMap.Refresh();
        }

        private ThinkGeo.MapSuite.Core.RectangleShape GetMaximumExtentFromLayers()
        {
            Collection<ThinkGeo.MapSuite.Core.BaseShape> bounds = new Collection<ThinkGeo.MapSuite.Core.BaseShape>();

            foreach (LayerOverlay overlay in this.wpfMap.Overlays.OfType<LayerOverlay>())
            {
                foreach (ThinkGeo.MapSuite.Core.Layer layer in overlay.Layers)
                {
                    layer.Open();
                    if (layer.HasBoundingBox == true)
                    {
                        bounds.Add(layer.GetBoundingBox());
                    }
                }
            }

            return ThinkGeo.MapSuite.Core.ExtentHelper.GetBoundingBoxOfItems(bounds);
        }

        private void removeButton_Click(object sender, RoutedEventArgs e)
        {
            if (restrictionOverlay != null)
            {
                this.wpfMap.Overlays.Remove(restrictionOverlay);
                //wpfMap.MapUnit = ThinkGeo.MapSuite.Core.GeographyUnit.Meter;
                this.removeButton.IsEnabled = false;
                this.layoutButton.IsEnabled = true;
                wpfMap.MapResizeMode = MapResizeMode.PreserveScaleAndCenter;
                this.wpfMap.Refresh();
            }
        }

        private void getExtentButton_Click(object sender, RoutedEventArgs e)
        {
            extentTextBlock.Text = GetMaximumExtentFromLayers().ToString();
        }

        private void Window_Loaded_1(object sender, RoutedEventArgs routedEventArgs)
        {
            #region CommentedOut
            //wpfMap.OverlayDrawing += wpfMap_OverlayDrawing;
            //wpfMap.CurrentExtent = new ThinkGeo.MapSuite.Core.RectangleShape(0, 0, 10, 20);
            //wpfMap.CurrentExtent = new RectangleShape(-0.5, 0.5, 1023.5, -767.5);
            //wpfMap.BackgroundOverlay.IsVisible = true;
            //wpfMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));
            #endregion

            wpfMap.MapResizeMode = MapResizeMode.PreserveScaleAndCenter;
            wpfMap.Visibility = System.Windows.Visibility.Visible;
            wpfMap.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //wpfMap.ZoomLevelSet
            ZoomLevelSet myZoomlevelSet = new ZoomLevelSet();
            myZoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(10000));
            myZoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(myZoomlevelSet.CustomZoomLevels[myZoomlevelSet.CustomZoomLevels.Count - 1].Scale / 2));
            myZoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(myZoomlevelSet.CustomZoomLevels[myZoomlevelSet.CustomZoomLevels.Count - 1].Scale / 2));
            myZoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(myZoomlevelSet.CustomZoomLevels[myZoomlevelSet.CustomZoomLevels.Count - 1].Scale / 2));
            myZoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(myZoomlevelSet.CustomZoomLevels[myZoomlevelSet.CustomZoomLevels.Count - 1].Scale / 2));
            myZoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(myZoomlevelSet.CustomZoomLevels[myZoomlevelSet.CustomZoomLevels.Count - 1].Scale / 2));
            //wpfMap.ZoomLevelSet = myZoomlevelSet;

            #region CommentedOut

            //ZoomLevelSet zoomSet = new ZoomLevelSet();
            //wpfMap.ZoomLevelSet = zoomSet;

            //FileStream fs = new FileStream(@"Chrysanthemum.jpg",FileMode.Open, FileAccess.Read);
            //BinaryReader br = new BinaryReader(fs);
            //byte[] imageContent = br.ReadBytes((int)fs.Length);
            //br.Close();
            //fs.Close();

            #endregion

            GdiPlusRasterLayer rasterLayer = new GdiPlusRasterLayer();
            rasterLayer.UpperThreshold = double.MaxValue;
            rasterLayer.LowerThreshold = 0;
            rasterLayer.IsGrayscale = false;
            ((GdiPlusRasterSource)rasterLayer.ImageSource).StreamLoading += (s, e) =>
            {
                if (e.StreamType == "Image File")
                {
                    Stream stream = new FileStream(@"Chrysanthemum.jpg", FileMode.Open, FileAccess.Read);
                    e.AlternateStream = stream;
                }
                else if (e.StreamType.Equals("World File", StringComparison.OrdinalIgnoreCase))
                {
                    Stream stream = new FileStream(@"Chrysanthemum.jpg", FileMode.Open, FileAccess.Read);

                    RetainableMemoryStream result = new RetainableMemoryStream();
                    using (StreamWriter writer = new StreamWriter(result))
                    {
                        double scaleX = 1.0;
                        double rotationY = 0.0;
                        double rotationX = 0.0;
                        double scaleY = -1.0;
                        double positionX = 0.0;
                        double positionY = 0.0;

                        writer.WriteLine(scaleX.ToString("R", CultureInfo.InvariantCulture));
                        writer.WriteLine(rotationY.ToString("R", CultureInfo.InvariantCulture));
                        writer.WriteLine(rotationX.ToString("R", CultureInfo.InvariantCulture));
                        writer.WriteLine(scaleY.ToString("R", CultureInfo.InvariantCulture));
                        writer.WriteLine(positionX.ToString("R", CultureInfo.InvariantCulture));
                        writer.WriteLine(positionY.ToString("R", CultureInfo.InvariantCulture));
                    }

                    result.Position = 0;
                    result.AllowDisposition = true;
                    e.AlternateStream = result;

                    //e.AlternateStream = new FileStream("Sapelo_2006_20080903T140150.jgw", FileMode.Open);//result;
                }
            };


            LayerOverlay layerOverLay = new LayerOverlay();
            layerOverLay.Drawing += layerOverLay_Drawing;
            layerOverLay.DrawingException += layerOverLay_DrawingException;
            layerOverLay.Layers.Add(rasterLayer);

            rasterLayer.Open();
            wpfMap.CurrentExtent = rasterLayer.GetBoundingBox();

            #region CommentedOut

            //Collection<BaseShape> bounds = new Collection<BaseShape>();
            //foreach (Layer layer in layerOverLay.Layers)
            //{
            //	layer.Open();
            //	if (layer.HasBoundingBox)
            //	{
            //		bounds.Add(layer.GetBoundingBox());
            //	}
            //}

            #endregion

            //RectangleShape restrictedExtentForPanning = new RectangleShape(-0.5, 0.5, 100.5, -100.5);
            //RectangleShape restrictedExtentForZoomOut = null;

            ////Gets the restricted extent for zooming out based on the restrcited extent for panning
            //restrictedExtentForZoomOut = (RectangleShape)restrictedExtentForPanning.CloneDeep();
            //restrictedExtentForZoomOut.ScaleUp(80);

            ////wpfMap.CurrentExtent = restrictedExtentForPanning;


            ////InMemoryFeatureLayer to show the restricted extent. It is just for display.

            //InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();

            //inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.Transparent,

            //                            GeoColor.StandardColors.Green, 3);

            //inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;



            //inMemoryFeatureLayer.Open();

            //inMemoryFeatureLayer.InternalFeatures.Add(new Feature(restrictedExtentForPanning));

            //inMemoryFeatureLayer.InternalFeatures.Add(new Feature(restrictedExtentForZoomOut));

            //inMemoryFeatureLayer.Close();

            #region CommentedOut
            //layerOverLay.Layers.Add(inMemoryFeatureLayer);
            //LayerOverlay sampleOverlay = new LayerOverlay();

            //sampleOverlay.Layers.Add(inMemoryFeatureLayer);

            //wpfMap.CurrentExtent = restrictedExtentForPanning;

            //wpfMap.Overlays.Add(sampleOverlay);

            //wpfMap.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(bounds);
            //wpfMap.MapResizeMode = MapResizeMode.PreserveScaleAndCenter;

            #endregion


            PointShape upperleft = rasterLayer.GetBoundingBox().UpperLeftPoint;
            PointShape lowerRight = upperleft.CloneDeep() as PointShape;
            lowerRight.TranslateByOffset(100, -100);
            RectangleShape restrictArea = new RectangleShape(upperleft, lowerRight);

            RestrictionLayer restrictionLayer = new RestrictionLayer();
            //maximumExtent.ScaleDown(50);
            //restrictionLayer.Transparency = 1.0f;
            restrictionLayer.Zones.Add(restrictArea);
            restrictionLayer.RestrictionMode = RestrictionMode.ShowZones;
            restrictionLayer.UpperScale = 250000000;
            restrictionLayer.LowerScale = double.MinValue;
            wpfMap.MapResizeMode = MapResizeMode.PreserveScaleAndCenter;
            this.layoutButton.IsEnabled = false;
            this.removeButton.IsEnabled = true;

            #region CommentedOut

            //this.wpfMap.CurrentScale = 1.0;
            //wpfMap.CurrentExtent = maximumExtent;
            //wpfMap.RestrictExtent = maximumExtent;
            //wpfMap.RestrictExtent = new RectangleShape(-0.5, 0.5, 100.5, -100.5); 
            //maxExtent = wpfMap.CurrentExtent;
            //wpfMap.CurrentExtentChanging += wpfMap_CurrentExtentChanging;
            //wpfMap.RestrictExtent = wpfMap.CurrentExtent;
            //wpfMap.MaximumScale = wpfMap.CurrentScale;
            //wpfMap.RestrictExtent = new RectangleShape(-0.5, 0.5, 50, -200.5);
            //wpfMap.CurrentExtent = new RectangleShape(-0.5, 0.5, 100, -767.5);

            #endregion

            restrictionOverlay = new LayerOverlay();
            restrictionOverlay.Layers.Add("RestrictionLayer", restrictionLayer);




            wpfMap.Overlays.Add(layerOverLay);
            wpfMap.Overlays.Add("RestrictionOverlay", restrictionOverlay);
            //wpfMap.CurrentExtent = GetMaximumExtentFromLayers().GetIntersection(restrictionLayer.Zones[0].GetBoundingBox());
            //wpfMap.RestrictExtent = wpfMap.CurrentExtent;
            wpfMap.Refresh();
        }

    }
}
